@@ -6,7 +6,7 @@ import os
 from os.path import exists
 import struct
 import sys
-from typing import IO, BinaryIO, Literal, TextIO
+from typing import IO, BinaryIO, Literal
 
 from configobj import ConfigObj, ConfigObjError
 from Cryptodome.Cipher import AES
@@ -23,7 +23,7 @@ def log(logger: logging.Logger, level: int, message: str) -> None:
     logger.log(level, message)
 
 
-def read_config_file(f: str | TextIO | TextIOWrapper, list_values: bool = True) -> ConfigObj | None:
+def read_config_file(f: str | IO[str], list_values: bool = True) -> ConfigObj | None:
     """Read a config file.
 
     *list_values* set to `True` is the default behavior of ConfigObj.
@@ -50,7 +50,7 @@ def read_config_file(f: str | TextIO | TextIOWrapper, list_values: bool = True)
     return config
 
 
-def get_included_configs(config_file: str | TextIOWrapper) -> list[str | TextIOWrapper]:
+def get_included_configs(config_file: str | IO[str]) -> list[str | IO[str]]:
     """Get a list of configuration files that are included into config_path
     with !includedir directive.
 
@@ -62,7 +62,7 @@ def get_included_configs(config_file: str | TextIOWrapper) -> list[str | TextIOW
     """
     if not isinstance(config_file, str) or not os.path.isfile(config_file):
         return []
-    included_configs: list[str | TextIOWrapper] = []
+    included_configs: list[str | IO[str]] = []
 
     try:
         with open(config_file) as f:
@@ -78,7 +78,7 @@ def get_included_configs(config_file: str | TextIOWrapper) -> list[str | TextIOW
     return included_configs
 
 
-def read_config_files(files: list[str | TextIOWrapper], list_values: bool = True) -> ConfigObj:
+def read_config_files(files: list[str | IO[str]], list_values: bool = True) -> ConfigObj:
     """Read and merge a list of config files."""
 
     config = create_default_config(list_values=list_values)
@@ -101,14 +101,15 @@ def read_config_files(files: list[str | TextIOWrapper], list_values: bool = True
 def create_default_config(list_values: bool = True) -> ConfigObj:
     import mycli
 
-    default_config_file = resources.open_text(mycli, "myclirc")
+    default_config_file = resources.files(mycli).joinpath("myclirc").open('r')
     return read_config_file(default_config_file, list_values=list_values)
 
 
 def write_default_config(destination: str, overwrite: bool = False) -> None:
     import mycli
 
-    default_config = resources.read_text(mycli, "myclirc")
+    with resources.files(mycli).joinpath("myclirc").open('r') as f:
+        default_config = f.read()
     destination = os.path.expanduser(destination)
     if not overwrite and exists(destination):
         return
